import os
import re

from freddist.command.install import install
from freddist.core import setup
from freddist.util import find_packages


PROJECT_NAME = 'stats_collector'


class FredStatsInstall(install):
    DEPS_PYMODULE = ('psycopg2',)
    DEPS_HELP = {'deb': {'psycopg2': 'python-psycopg2'}}

    def update_config_path(self, filename):
        """
        Update config path variable.
        """
        content = open(filename).read()
        pattern = re.compile(r'^CONFIG_PATH\s*=.*$', re.MULTILINE)
        content = pattern.sub('CONFIG_PATH = "%s"' % self.expand_filename('$sysconf/fred'), content)
        open(filename, 'w').write(content)
        self.announce("File '%s' was updated" % filename)


def main():
    srcdir = os.path.dirname(os.path.abspath(__file__))
    packages = find_packages(srcdir)

    data_files = [('$doc', ['doc/README.txt']),
                  ('$sysconf/fred', ['stats_collector_cfg.py'])]

    setup(name=PROJECT_NAME,
          description='Fred statistics add data script',
          author='Tomas Divis, CZ.NIC',
          author_email='tomas.divis@nic.cz',
          url='http://www.nic.cz/',
          license='GNU GPL',
          platforms=['posix'],
          long_description='Fred statistics script for adding new daily data to database.',
          packages=packages,
          data_files=data_files,
          scripts=['collect_stats.py'],
          modify_files={'$scripts/collect_stats.py': 'update_config_path'},
          cmdclass={'install': FredStatsInstall})


if __name__ == '__main__':
    main()
