import sys
from argparse import ArgumentParser
import datetime

def parse_date(date_str):
    return datetime.date(*[int(item) for item in date_str.split('-')])

def parse_string_list(string_list):
    return string_list.split(',')

parser = ArgumentParser(description='Run statistic group for a given date.')

parser.add_argument(dest='run_date', metavar='YYYY-MM-DD', type=parse_date, nargs='?',
                    default=datetime.date.today() + datetime.timedelta(-1),
                    help='Run for specified date (yesterday is used if ommited)')
parser.add_argument(dest='run_date_end', metavar='YYYY-MM-DD', type=parse_date, nargs='?',
                    help='If specified, collector will run in day loop for run_date in interval <run_date, run_date_end)')
group = parser.add_mutually_exclusive_group()
group.add_argument('-s', '--statistic-groups', dest='stat_groups', metavar='stat1,stat2', type=parse_string_list,
                    default=None,
                    help='Comma separated list of statistics groups')
group.add_argument('-n', '--statistics-names', dest='stat_names', metavar='name_1,name_2,...,name_n', type=parse_string_list,
                    default=None,
                    help='Comma separated list of statistics names to run if present in specified groups (mainly for developers)')
group.add_argument('-l', '--list', dest='stat_list', action='store_true',
                    default=False,
                    help='List all available statistics and exit')
parser.add_argument('--force-overwrite', dest='force_overwrite', action='store_true',
                    default=False,
                    help='Force to overwrite output files when exist')

args = parser.parse_args()
