from stats_collector.stats import Statistics
from stats_collector import FRED_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByRegistrarAndDNSSEC(Statistics):
    sql = '''
        SELECT
            q.d_status,
            baz.handle,
            CASE
                WHEN q.d_status = 'dnssec' THEN baz.dnssec_count
                WHEN q.d_status = 'no_dnssec' THEN baz.no_dnssec_count
            END
          FROM
            (VALUES ('dnssec'),('no_dnssec')) as q (d_status)
            CROSS JOIN (
                SELECT
                    r.handle,
                    coalesce(foo.dnssec_count, 0) AS dnssec_count,
                    coalesce(foo.no_dnssec_count, 0) AS no_dnssec_count
                  FROM
                    registrar r
                    LEFT JOIN (
                        SELECT
                            obj.clid AS registrar_id,
                            z.id AS zone_id,
                            sum(CASE WHEN d.keyset IS NULL THEN 1 ELSE 0 END) AS no_dnssec_count,
                            sum(CASE WHEN d.keyset IS NOT NULL THEN 1 ELSE 0 END) AS dnssec_count,
                            count(d.id) AS domain_count
                          FROM
                            object_registry o
                            JOIN domain_history d ON (d.id = o.id)
                            JOIN zone z ON (d.zone = z.id)
                            JOIN history h ON (h.id = d.historyid)
                            JOIN object_history obj ON (obj.historyid = h.id)
                          WHERE
                            o.type = 3
                            AND (o.crdate < %(midnight)s
                                AND (o.erdate IS NULL OR o.erdate >= %(midnight)s))
                            AND o.name NOT LIKE '%(monitoring_domain)s'
                            AND z.fqdn = 'cz'
                            AND h.valid_from < %(midnight)s
                                AND (h.valid_to IS NULL OR h.valid_to >= %(midnight)s)
                          GROUP BY 1, 2
                        ) AS foo ON foo.registrar_id = r.id
                    LEFT JOIN registrarinvoice ri ON ri.registrarid = r.id
                        AND ri.fromdate <= %(midnight)s
                        AND (ri.todate >= %(midnight)s OR ri.todate IS NULL)
                        AND ri.zone = (SELECT id FROM zone WHERE fqdn = 'cz')
                    WHERE
                        foo.domain_count > 0 OR ri.id IS NOT NULL
                    ORDER BY 1) AS baz
          ORDER BY 2, 1
    '''
    name = 'dnssec_by_registrar'
    stat_db = FRED_DB

