from stats_collector.stats import Statistics
from stats_collector import FRED_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByDNSSEC(Statistics):
    sql = '''
        SELECT
            CASE
                WHEN D.keyset IS NOT NULL then 'dnssec'
                ELSE 'no_dnssec'
            END,
            COUNT(*)
        FROM
            object_registry O
            JOIN domain_history D ON (D.id = O.id)
            JOIN zone Z ON (D.zone = Z.id)
            JOIN history H ON (H.id = D.historyid)
        WHERE type = 3
            AND (crdate <= %(midnight)s AND (erdate IS NULL OR erdate >= %(midnight)s))
            AND name NOT LIKE '%(monitoring_domain)s'
            AND H.valid_from < %(midnight)s AND (H.valid_to IS NULL OR H.valid_to >= %(midnight)s)
            AND Z.fqdn = 'cz'
        GROUP BY 1

    '''
    name = 'domains_by_dnssec'
    stat_db = FRED_DB

