from stats_collector.stats import Statistics
from stats_collector import FRED_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByRegistrar(Statistics):
    sql = '''
        SELECT r.handle, coalesce(foo.domain_count, 0)
            FROM
                registrar r
                LEFT JOIN (
                    SELECT R.handle, COUNT(*) as domain_count
                        FROM
                            object_registry O
                            JOIN domain_history D ON (D.id = O.id)
                            JOIN zone Z ON (D.zone = Z.id)
                            JOIN history H ON (H.id = D.historyid)
                            JOIN object_history OBJ ON (OBJ.historyid = H.id)
                            JOIN registrar R ON (OBJ.clid = R.id)
                      WHERE
                        type = 3
                        AND (crdate < %(midnight)s AND (erdate IS NULL OR erdate >= %(midnight)s))
                        AND O.name NOT LIKE '%(monitoring_domain)s'
                        AND Z.fqdn = 'cz'
                        AND H.valid_from < %(midnight)s AND (H.valid_to IS NULL OR H.valid_to >= %(midnight)s)
                    GROUP BY R.handle
                    ORDER BY R.handle
                ) AS foo ON foo.handle = r.handle
                LEFT JOIN registrarinvoice ri ON ri.registrarid = r.id
                        AND ri.fromdate <= %(midnight)s
                        AND (ri.todate >= %(midnight)s OR ri.todate IS NULL)
                        AND ri.zone = (SELECT id FROM zone WHERE fqdn = 'cz')
              WHERE
                foo.domain_count > 0 OR ri.id IS NOT NULL
        ORDER BY 1
    '''
    name = 'domains_by_registrar'
    stat_db = FRED_DB

