from stats_collector.stats import Statistics
from stats_collector import FRED_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByStatusTotal(Statistics):
    sql = '''
        SELECT 'total', COUNT(*)
        FROM
            object_registry O
            JOIN domain_history D ON (D.historyid = O.crhistoryid)
            JOIN zone Z ON (D.zone = Z.id)
        WHERE type = 3
            AND (crdate < %(midnight)s AND (erdate IS NULL OR erdate >= %(midnight)s))
            AND name NOT LIKE '%(monitoring_domain)s'
            AND Z.fqdn = 'cz'
    '''
    name = 'domains_by_status'
    stat_db = FRED_DB

@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByStatusInZone(Statistics):
    sql = '''
        SELECT 'in_zone', COUNT(*)
        FROM
            object_registry O
            JOIN domain_history D ON (D.historyid = O.crhistoryid)
            JOIN zone Z ON (D.zone = Z.id)
            LEFT JOIN object_state s ON (
                O.id = S.object_id
                AND S.state_id=15 AND S.valid_from < %(midnight)s
                AND (S.valid_to ISNULL OR S.valid_to >= %(midnight)s))
        WHERE
            crdate < %(midnight)s
            AND (erdate IS NULL OR erdate >= %(midnight)s)
            AND S.id IS NULL AND O.type=3
            AND Z.fqdn = 'cz'
    '''
    name = 'domains_by_status'
    stat_db = FRED_DB

@stat_registry.register(FRED_DAILY_GROUP)
class DomainsByStatusInProtection(Statistics):
    sql = '''
        SELECT 'in_protection_period', COUNT(*)
        FROM
            object_registry O
            JOIN domain_history D ON (D.historyid = O.crhistoryid)
            JOIN zone Z ON (D.zone = Z.id)
            JOIN object_state S ON (
                O.id = S.object_id
                AND S.state_id = 9 and S.valid_from < %(midnight)s
                AND (S.valid_to IS NULL OR S.valid_to >= %(midnight)s))
        WHERE
            crdate < %(midnight)s
            AND (erdate isnull or erdate >= %(midnight)s)
            AND O.type=3
            AND Z.fqdn = 'cz'
    '''
    name = 'domains_by_status'
    stat_db = FRED_DB

