from stats_collector.stats import Statistics
from stats_collector import FRED_DB, LOG_DB
from stats_collector.stats.stat_registry import stat_registry
from ..groups import FRED_DAILY_GROUP


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainCreate(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'BillingFailure' THEN 'billing_failure'
                WHEN rc.name = 'ObjectExists' THEN 'object_exists'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainCreate'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainCreate'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB


@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainUpdate(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainUpdate'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainUpdate'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainRenew(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'BillingFailure' THEN 'billing_failure'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainRenew'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainRenew'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainTransfer(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'AuthorizationError' THEN 'authorization_error'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainTransfer'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainTransfer'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainCheck(Statistics):
    sql = '''
        SELECT
            rt.name,
            'success',
            COUNT(*)
          FROM
            request_type rt
            JOIN request r ON r.request_type_id = rt.id
            JOIN result_code rc ON rc.id = r.result_code_id
          WHERE
            %(log_req_cond_epp)s
            AND rt.name = 'DomainCheck'
            AND rc.name = 'CommandCompletedSuccessfully'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainInfo(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainInfo'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainInfo'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationByResultDomainSendAuthInfo(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                WHEN rc.name = 'ObjectDoesNotExist' THEN 'object_does_not_exist'
                WHEN rc.name = 'ObjectStatusProhibitsOperation' THEN 'object_status_prohibits_operation'
                ELSE 'fail'
            END,
            COUNT(foo.r_id)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND rt.name = 'DomainSendAuthInfo'
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'DomainSendAuthInfo'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultDomainDelete(Statistics):
    sql = '''
        SELECT
            'DomainDelete', 'success_registry', COUNT(*)
          FROM
              object_registry O
              JOIN domain_history D ON (D.historyid = O.crhistoryid)
              JOIN zone Z ON (D.zone = Z.id)
              JOIN object_state S ON (O.id = S.object_id)
          WHERE
              erdate BETWEEN '%(date)s'::timestamp AND %(midnight)s
              -- deleteCandidate which was not canceled (e.g. unblocking after law suits)
              AND S.state_id = 17 AND valid_to IS NULL
              AND O.type = 3
              AND Z.fqdn = 'cz'
        UNION
        SELECT
            'DomainDelete', 'success_registrars', COUNT(*)
          FROM
            object_registry oreg
            JOIN domain_history dh ON dh.historyid = oreg.crhistoryid
            JOIN zone z ON z.id = dh.zone
            LEFT JOIN object_state os ON os.object_id = oreg.id AND os.state_id = 17
          WHERE
            erdate BETWEEN '%(date)s'::timestamp AND %(midnight)s
            AND oreg.erdate IS NOT NULL
            AND oreg.type = 3
            AND z.fqdn = 'cz'
            AND os.id IS NULL
    '''
    name = 'request_type_by_result_count'
    stat_db = FRED_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultContact(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        rt.name = 'ContactCreate'
                        OR rt.name = 'ContactUpdate'
                        OR rt.name = 'ContactTransfer'
                        OR rt.name = 'ContactCheck'
                        OR rt.name = 'ContactInfo'
                        OR rt.name = 'ContactSendAuthInfo'
                        OR rt.name = 'ContactDelete'
                    )
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'ContactCreate'
                OR rt.name = 'ContactUpdate'
                OR rt.name = 'ContactTransfer'
                OR rt.name = 'ContactCheck'
                OR rt.name = 'ContactInfo'
                OR rt.name = 'ContactSendAuthInfo'
                OR rt.name = 'ContactDelete'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultNSSet(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        rt.name = 'NSsetCreate'
                        OR rt.name = 'NSsetUpdate'
                        OR rt.name = 'NSsetTransfer'
                        OR rt.name = 'NSsetCheck'
                        OR rt.name = 'NSsetInfo'
                        OR rt.name = 'NSsetSendAuthInfo'
                        OR rt.name = 'NSsetDelete'
                    )
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'NSsetCreate'
                OR rt.name = 'NSsetUpdate'
                OR rt.name = 'NSsetTransfer'
                OR rt.name = 'NSsetCheck'
                OR rt.name = 'NSsetInfo'
                OR rt.name = 'NSsetSendAuthInfo'
                OR rt.name = 'NSsetDelete'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

@stat_registry.register(FRED_DAILY_GROUP)
class OperationbyResultKeySet(Statistics):
    sql = '''
        SELECT
            rt.name,
            CASE
                WHEN rc.name = 'CommandCompletedSuccessfully' THEN 'success'
                ELSE 'fail'
            END,
            COALESCE(COUNT(foo.r_id), 0)
          FROM
            request_type rt
            LEFT JOIN result_code rc ON rc.service_id = rt.service_id AND rt.service_id = 3
            LEFT JOIN (
                SELECT
                    rt.name as rt_name,
                    rc.name as rc_name,
                    r.id as r_id
                  FROM
                    request_type rt
                    JOIN request r ON r.request_type_id = rt.id
                    JOIN result_code rc ON rc.id = r.result_code_id
                  WHERE
                    %(log_req_cond_epp)s
                    AND (
                        rt.name = 'KeysetCreate'
                        OR rt.name = 'KeysetUpdate'
                        OR rt.name = 'KeysetTransfer'
                        OR rt.name = 'KeysetCheck'
                        OR rt.name = 'KeysetInfo'
                        OR rt.name = 'KeySetSendAuthInfo'
                        OR rt.name = 'KeysetDelete'
                    )
                ) foo ON foo.rt_name = rt.name AND foo.rc_name = rc.name
          WHERE
                rt.name = 'KeysetCreate'
                OR rt.name = 'KeysetUpdate'
                OR rt.name = 'KeysetTransfer'
                OR rt.name = 'KeysetCheck'
                OR rt.name = 'KeysetInfo'
                OR rt.name = 'KeySetSendAuthInfo'
                OR rt.name = 'KeysetDelete'
          GROUP BY 1, 2
          ORDER BY 1, 2
    '''
    name = 'request_type_by_result_count'
    stat_db = LOG_DB

